/*
Copyright 2019 Rancher Labs.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"time"

	v1 "github.com/rancher/rio/pkg/apis/admin.rio.cattle.io/v1"
	scheme "github.com/rancher/rio/pkg/generated/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// SystemStacksGetter has a method to return a SystemStackInterface.
// A group's client should implement this interface.
type SystemStacksGetter interface {
	SystemStacks() SystemStackInterface
}

// SystemStackInterface has methods to work with SystemStack resources.
type SystemStackInterface interface {
	Create(*v1.SystemStack) (*v1.SystemStack, error)
	Update(*v1.SystemStack) (*v1.SystemStack, error)
	Delete(name string, options *metav1.DeleteOptions) error
	DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error
	Get(name string, options metav1.GetOptions) (*v1.SystemStack, error)
	List(opts metav1.ListOptions) (*v1.SystemStackList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.SystemStack, err error)
	SystemStackExpansion
}

// systemStacks implements SystemStackInterface
type systemStacks struct {
	client rest.Interface
}

// newSystemStacks returns a SystemStacks
func newSystemStacks(c *AdminV1Client) *systemStacks {
	return &systemStacks{
		client: c.RESTClient(),
	}
}

// Get takes name of the systemStack, and returns the corresponding systemStack object, and an error if there is any.
func (c *systemStacks) Get(name string, options metav1.GetOptions) (result *v1.SystemStack, err error) {
	result = &v1.SystemStack{}
	err = c.client.Get().
		Resource("systemstacks").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of SystemStacks that match those selectors.
func (c *systemStacks) List(opts metav1.ListOptions) (result *v1.SystemStackList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.SystemStackList{}
	err = c.client.Get().
		Resource("systemstacks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested systemStacks.
func (c *systemStacks) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Resource("systemstacks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a systemStack and creates it.  Returns the server's representation of the systemStack, and an error, if there is any.
func (c *systemStacks) Create(systemStack *v1.SystemStack) (result *v1.SystemStack, err error) {
	result = &v1.SystemStack{}
	err = c.client.Post().
		Resource("systemstacks").
		Body(systemStack).
		Do().
		Into(result)
	return
}

// Update takes the representation of a systemStack and updates it. Returns the server's representation of the systemStack, and an error, if there is any.
func (c *systemStacks) Update(systemStack *v1.SystemStack) (result *v1.SystemStack, err error) {
	result = &v1.SystemStack{}
	err = c.client.Put().
		Resource("systemstacks").
		Name(systemStack.Name).
		Body(systemStack).
		Do().
		Into(result)
	return
}

// Delete takes name of the systemStack and deletes it. Returns an error if one occurs.
func (c *systemStacks) Delete(name string, options *metav1.DeleteOptions) error {
	return c.client.Delete().
		Resource("systemstacks").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *systemStacks) DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Resource("systemstacks").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched systemStack.
func (c *systemStacks) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.SystemStack, err error) {
	result = &v1.SystemStack{}
	err = c.client.Patch(pt).
		Resource("systemstacks").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
