/*
Copyright 2019 Rancher Labs.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v3

import (
	"time"

	v3 "github.com/rancher/rio/pkg/apis/management.cattle.io/v3"
	scheme "github.com/rancher/rio/pkg/generated/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// SettingsGetter has a method to return a SettingInterface.
// A group's client should implement this interface.
type SettingsGetter interface {
	Settings() SettingInterface
}

// SettingInterface has methods to work with Setting resources.
type SettingInterface interface {
	Create(*v3.Setting) (*v3.Setting, error)
	Update(*v3.Setting) (*v3.Setting, error)
	Delete(name string, options *v1.DeleteOptions) error
	DeleteCollection(options *v1.DeleteOptions, listOptions v1.ListOptions) error
	Get(name string, options v1.GetOptions) (*v3.Setting, error)
	List(opts v1.ListOptions) (*v3.SettingList, error)
	Watch(opts v1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v3.Setting, err error)
	SettingExpansion
}

// settings implements SettingInterface
type settings struct {
	client rest.Interface
}

// newSettings returns a Settings
func newSettings(c *ManagementV3Client) *settings {
	return &settings{
		client: c.RESTClient(),
	}
}

// Get takes name of the setting, and returns the corresponding setting object, and an error if there is any.
func (c *settings) Get(name string, options v1.GetOptions) (result *v3.Setting, err error) {
	result = &v3.Setting{}
	err = c.client.Get().
		Resource("settings").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of Settings that match those selectors.
func (c *settings) List(opts v1.ListOptions) (result *v3.SettingList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v3.SettingList{}
	err = c.client.Get().
		Resource("settings").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested settings.
func (c *settings) Watch(opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Resource("settings").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a setting and creates it.  Returns the server's representation of the setting, and an error, if there is any.
func (c *settings) Create(setting *v3.Setting) (result *v3.Setting, err error) {
	result = &v3.Setting{}
	err = c.client.Post().
		Resource("settings").
		Body(setting).
		Do().
		Into(result)
	return
}

// Update takes the representation of a setting and updates it. Returns the server's representation of the setting, and an error, if there is any.
func (c *settings) Update(setting *v3.Setting) (result *v3.Setting, err error) {
	result = &v3.Setting{}
	err = c.client.Put().
		Resource("settings").
		Name(setting.Name).
		Body(setting).
		Do().
		Into(result)
	return
}

// Delete takes name of the setting and deletes it. Returns an error if one occurs.
func (c *settings) Delete(name string, options *v1.DeleteOptions) error {
	return c.client.Delete().
		Resource("settings").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *settings) DeleteCollection(options *v1.DeleteOptions, listOptions v1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Resource("settings").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched setting.
func (c *settings) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v3.Setting, err error) {
	result = &v3.Setting{}
	err = c.client.Patch(pt).
		Resource("settings").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
